#ifndef __HWESP_GLOBAL_H__
#define __HWESP_GLOBAL_H__
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/usb/input.h>
/* device IDs */
#define STYLUS_DEVICE_ID	0x02
#define TOUCH_DEVICE_ID		0x03
#define CURSOR_DEVICE_ID	0x06
#define ERASER_DEVICE_ID	0x0A
#define PAD_DEVICE_ID		0x0F

struct hanwang {
	unsigned char *data;
	dma_addr_t data_dma;
	struct input_dev *dev;
	struct usb_device *usbdev;
	struct urb *irq;
	const struct hanwang_features *features;
	unsigned int current_tool;
	unsigned int current_id;
	char name[64];
	char phys[32];
};

struct hanwang_features {
	unsigned short pid;
	char *name;
//	enum hanwang_tablet_type type;
	int pkg_len;
	int max_x;
	int max_y;
	int max_tilt_x;
	int max_tilt_y;
	int max_pressure;
};

#define USB_VENDOR_ID_HANWANG		0x0b57
#define HANWANG_TABLET_INT_CLASS	0x0003
#define HANWANG_TABLET_INT_SUB_CLASS	0x0001
#define HANWANG_TABLET_INT_PROTOCOL	0x0000

/* match vendor and interface info  */
#define HANWANG_TABLET_DEVICE(vend, prod,cl, sc) \
	.match_flags = USB_DEVICE_ID_MATCH_VENDOR \ 
        | USB_DEVICE_ID_MATCH_PRODUCT \
		| USB_DEVICE_ID_MATCH_INT_CLASS\
        | USB_DEVICE_ID_MATCH_INT_SUBCLASS, \
	.idVendor = (vend), \
    .idProduct = (prod), \
	.bInterfaceClass = (cl), \
	.bInterfaceSubClass = (sc), 

static const struct usb_device_id hanwang_ids[] = {
	{ HANWANG_TABLET_DEVICE(USB_VENDOR_ID_HANWANG, 0x855C,HANWANG_TABLET_INT_CLASS,
		HANWANG_TABLET_INT_SUB_CLASS) },
    { HANWANG_TABLET_DEVICE(0x0b57, 0x880c,3, 1)},
{ HANWANG_TABLET_DEVICE(0x0b57, 0x880b,3, 1)},
{ HANWANG_TABLET_DEVICE(0x0b57, 0x8811,3, 1)},
{ HANWANG_TABLET_DEVICE(0x0b57, 0x8824,3, 1)},
{ HANWANG_TABLET_DEVICE(0x0b57, 0x8825,3, 1)},
{}
};

int  open(struct input_dev *dev);
bool get_features(unsigned short idProduct, struct hanwang_features *features);
void irq(int urbstatus,struct urb * urb,struct usb_device *dev, unsigned short pid,struct input_dev *input_dev, unsigned char *data);
int probe(struct input_dev *input_dev,unsigned short pid,unsigned long* evbit,unsigned long* absbit,unsigned long* keybit);
void hanwang_parse_packet(struct usb_device* dev,unsigned short pid,struct input_dev *input_dev,unsigned char *data);

#endif // HWESP_GLOBAL_H
